class ElasticSearchError(Exception):
    """ Base class for all ElasticSearch error classes.
        All exceptions raised by the benchmark runner library should inherit from this class. """
    pass


class ElasticSearchDataNotUploaded(ElasticSearchError):
    """This exception return elastic search not uploaded error"""
    def __init__(self):
        self.message = f'Data did not upload to elastic search'
        super(ElasticSearchDataNotUploaded, self).__init__(self.message)



import os


def is_empty_file(file_path):
    """
    This method check for empty file and raise as exception
    @param file_path:
    @return:
    """
    if os.stat(file_path).st_size == 0:
        raise Exception(f'File is empty: {file_path}')


def get_policies(file_type: str = '.py', exclude_policies: list = None):
    """
    This method return a list of policies name without extension, that can filter by type
    @return: list of custodian policies name
    """
    exclude_policies = [] if not exclude_policies else exclude_policies
    custodian_policies = []
    root_folder = os.path.dirname(os.path.dirname(os.path.dirname(__file__)))
    policies_path = os.path.join(root_folder, 'policy', 'aws')
    for (_, _, filenames) in os.walk(policies_path):
        for filename in filenames:
            if not filename.startswith('__') and filename.endswith(file_type):
                if filename.split('.')[0] not in exclude_policies:
                    if not file_type:
                        custodian_policies.append(os.path.splitext(filename)[0])
                    elif file_type and file_type in filename:
                        custodian_policies.append(os.path.splitext(filename)[0])
    return custodian_policies



from cloud_governance.policy.helpers.aws.aws_policy_operations import AWSPolicyOperations


class IpUnattached(AWSPolicyOperations):
    """
    Fetched the Unused elastic_ips( based on network interface Id) and delete it after 7 days of unused,
    alert user after 4 days of unused elastic_ip.
    """

    RESOURCE_ACTION = "Delete"

    def __init__(self):
        super().__init__()

    def run_policy_operations(self):
        """
        This method returns the list of unattached IPV4 addresses
        :return:
        :rtype:
        """
        unit_price = self._resource_pricing.get_eip_unit_price()
        addresses = self._ec2_operations.get_elastic_ips()
        active_cluster_ids = self._get_active_cluster_ids()
        unattached_addresses = []
        for address in addresses:
            tags = address.get('Tags', [])
            cleanup_result = False
            ip_not_used = False
            resource_id = address.get('AllocationId')
            cluster_tag = self._get_cluster_tag(tags=address.get('Tags'))
            if cluster_tag not in active_cluster_ids:
                if not address.get('NetworkInterfaceId'):
                    cleanup_days = self.get_clean_up_days_count(tags=tags)
                    ip_not_used = True
                    cleanup_result = self.verify_and_delete_resource(resource_id=resource_id, tags=tags,
                                                                     clean_up_days=cleanup_days)
                    resource_data = self._get_es_schema(resource_id=resource_id,
                                                        user=self.get_tag_name_from_tags(tags=tags, tag_name='User'),
                                                        skip_policy=self.get_skip_policy_value(tags=tags),
                                                        cleanup_days=cleanup_days, dry_run=self._dry_run,
                                                        name=self.get_tag_name_from_tags(tags=tags, tag_name='Name'),
                                                        region=self._region,
                                                        cleanup_result=str(cleanup_result),
                                                        resource_action=self.RESOURCE_ACTION,
                                                        cloud_name=self._cloud_name,
                                                        resource_type='PublicIPv4',
                                                        unit_price=unit_price,
                                                        resource_state='disassociated' if not cleanup_result else "Deleted"
                                                        )
                    unattached_addresses.append(resource_data)
                else:
                    cleanup_days = 0
                if not cleanup_result:
                    if self.get_tag_name_from_tags(tags, tag_name='DaysCount') or ip_not_used:
                        self.update_resource_day_count_tag(resource_id=resource_id, cleanup_days=cleanup_days, tags=tags)

        return unattached_addresses



from ast import literal_eval

from cloud_governance.common.elasticsearch.elasticsearch_operations import ElasticSearchOperations
from cloud_governance.common.logger.init_logger import logger
from cloud_governance.common.mails.mail_message import MailMessage
from cloud_governance.common.mails.postfix import Postfix
from cloud_governance.main.environment_variables import environment_variables


class ElasticUpload:

    def __init__(self):
        self.__environment_variables_dict = environment_variables.environment_variables_dict
        self.es_host = self.__environment_variables_dict.get('es_host', '')
        self.__es_port = self.__environment_variables_dict.get('es_port', '')
        self.es_index = self.__environment_variables_dict.get('es_index', '')
        self.account = self.__environment_variables_dict.get('account', '').upper().replace('OPENSHIFT-', '').strip()
        self.special_user_mails = self.__environment_variables_dict.get('special_user_mails', '{}')
        self.postfix_mail = Postfix()
        self.mail_message = MailMessage()
        self.elastic_search_operations = ElasticSearchOperations(es_host=self.es_host, es_port=self.__es_port) if self.es_host else None

    def es_upload_data(self, items: list, es_index: str = '', **kwargs):
        """
        This method upload data to elastic search
        @param items:
        @param es_index:
        @return:
        """
        try:
            if not es_index:
                es_index = self.es_index
            count = 0
            for item in items:
                if not item.get('Account'):
                    item['Account'] = kwargs.get('Account') if kwargs.get('Account') else self.account
                if kwargs.get('set_index'):
                    self.elastic_search_operations.upload_to_elasticsearch(index=es_index, data=item, id=item[kwargs.get('set_index')])
                else:
                    self.elastic_search_operations.upload_to_elasticsearch(index=es_index, data=item)
                count += 1
            if count > 0 and len(items) > 0:
                logger.warn(f'Data Uploaded to {es_index} successfully, Total data: {count}')
        except Exception as err:
            logger.error(f'Error raised {err}')

    def literal_eval(self, data: any):
        """
        This method convert string object into its original datatype
        ex: "{'Project': 'Cloud-Governance'}" --> {'Project': 'Cloud-Governance'}
        @param data:
        @return:
        """
        if data:
            return literal_eval(data)
        return data



import tempfile

import pandas as pd
import typeguard

from cloud_governance.common.clouds.aws.s3.s3_operations import S3Operations
from cloud_governance.common.logger.logger_time_stamp import logger_time_stamp


class PandasOperations:
    """
    This class performs the pandas operations
    """
    CHUNK_SIZE = 5000

    def __init__(self, region_name: str = 'us-east-1'):
        self.__s3_operations = S3Operations(region_name=region_name)

    @typeguard.typechecked
    @logger_time_stamp
    def get_dataframe_from_csv_file(self, file_path: str):
        """
        This method returns the pandas dataframe from the csv file
        :param file_path:
        :return:
        """
        dataframes = []
        for data_chunk in pd.read_csv(filepath_or_buffer=file_path, chunksize=self.CHUNK_SIZE):
            dataframes.append(data_chunk)
        dataframe = pd.concat(dataframes, ignore_index=True)
        return dataframe

    @typeguard.typechecked
    @logger_time_stamp
    def get_dataframe_from_s3_file(self, bucket: str, key: str, download_file: str):
        """
        This method returns the pandas dataframe from the s3 file
        :return:
        """
        if not self.__s3_operations.file_exist(bucket=bucket, key=key, file_name=download_file):
            raise FileNotFoundError(f"{key}/{download_file} path is not exists else verify your credentials")
        with tempfile.NamedTemporaryFile(suffix='.csv', mode='w') as file_name:
            self.__s3_operations.download_file(bucket=bucket, key=key, download_file=download_file,
                                               file_name_path=file_name.name)
            return self.get_dataframe_from_csv_file(file_path=file_name.name)

    @typeguard.typechecked
    @logger_time_stamp
    def get_data_dictonary_from_dataframe(self, dataframe: pd.DataFrame):
        """
        This method returns the dataframe format to dictonary order
        :param dataframe:
        :return:
        """
        return dataframe.to_dict(orient='records')



import datetime

import pytz
from azure.mgmt.costmanagement.models import QueryGrouping, QueryFilter, QueryComparisonExpression

from cloud_governance.common.clouds.azure.cost_management.cost_management_operations import CostManagementOperations
from cloud_governance.common.clouds.azure.subscriptions.azure_operations import AzureOperations
from cloud_governance.common.elasticsearch.elastic_upload import ElasticUpload
from cloud_governance.common.google_drive.google_drive_operations import GoogleDriveOperations
from cloud_governance.common.google_drive.upload_to_gsheet import UploadToGsheet
from cloud_governance.common.logger.init_logger import logger
from cloud_governance.common.logger.logger_time_stamp import logger_time_stamp
from cloud_governance.main.environment_variables import environment_variables


class CostBillingReports:
    """
    This class is responsible for generation cost billing report for Budget, Actual, Forecast
    """

    def __init__(self):
        self.__environment_variables_dict = environment_variables.environment_variables_dict
        self.__total_account = self.__environment_variables_dict.get('TOTAL_ACCOUNTS', '')
        self.azure_operations = AzureOperations()
        self.cost_mgmt_operations = CostManagementOperations()
        self.elastic_upload = ElasticUpload()
        self.gdrive_operations = GoogleDriveOperations()
        self.__gsheet_id = self.__environment_variables_dict.get('SPREADSHEET_ID')
        self.update_to_gsheet = UploadToGsheet()
        self.__cost_center, self.__allocated_budget, self.__years, self.__owner = self.update_to_gsheet.get_cost_center_budget_details(account_id=self.azure_operations.subscription_id, dir_path='/tmp')
        self.__common_data = self.get_common_data()

    def get_common_data(self):
        """
        This method gives the common data to added
        @return:
        """
        upload_data = {}
        upload_data['AccountId'] = self.azure_operations.subscription_id
        upload_data['Account'] = self.azure_operations.account_name+'-'+self.azure_operations.cloud_name.title()
        upload_data['Budget'] = 0
        upload_data['AllocatedBudget'] = 0
        upload_data['CostCenter'] = int(self.__cost_center)
        upload_data['CloudName'] = self.azure_operations.cloud_name
        upload_data['Owner'] = self.__owner
        return upload_data

    @logger_time_stamp
    def get_data_from_costs(self, cost_data_rows: list, cost_data_columns: list, cost_type: str, cost_billing_data: dict, subscription_id: str = '', account_name: str = '', cost_center: int = 0):
        """
        This method get the data from the cost
        @param cost_billing_data:
        @param cost_type:
        @param cost_data_rows:
        @param cost_data_columns:
        @param subscription_id:
        @return:
        """
        common_data = self.__common_data
        index_id = ''
        common_data["AccountId"] = subscription_id
        if account_name:
            common_data['Account'] = account_name
        if cost_center > 0:
            common_data['CostCenter'] = cost_center
        if subscription_id:
            cost_center, allocated_budget, years, owner = self.update_to_gsheet.get_cost_center_budget_details(account_id=subscription_id, dir_path='/tmp')
            if cost_center:
                common_data['CostCenter'] = int(cost_center)
                common_data['Owner'] = owner
            else:
                common_data['Owner'] = owner
        else:
            allocated_budget, years = self.__allocated_budget, self.__years
        for index, item in enumerate(cost_data_rows):
            cost_data = {}
            start_date = ''
            for key, column in enumerate(cost_data_columns):
                if column.get('type') == 'Number':
                    if cost_type == 'Forecast':
                        cost_data[item[2]] = round(item[key], 2)
                    else:
                        cost_data[cost_type] = round(item[key], 2)
                elif column.get('name') == 'SubscriptionName':
                    common_data['Account'] = item[key]
                elif column.get('name') == 'SubscriptionId':
                    common_data['AccountId'] = item[key]
                    cost_center, allocated_budget, years, owner = self.update_to_gsheet.get_cost_center_budget_details(account_id=item[key], dir_path='/tmp')
                    if cost_center:
                        common_data['CostCenter'] = int(cost_center)
                        common_data['Owner'] = owner
                    else:
                        common_data['Owner'] = owner
                else:
                    if column.get('type') == 'Datetime':
                        start_date = item[key].split('T')[0]
            index_id = f'{start_date}-{common_data["AccountId"]}'
            if index_id not in cost_billing_data:
                if start_date.split('-')[0] in years:
                    if 'Budget' in common_data:
                        common_data.pop('Budget')
                    if 'AllocatedBudget' in common_data:
                        common_data.pop('AllocatedBudget')
                cost_data['Budget'] = round(allocated_budget / 12, 3)
                cost_data['AllocatedBudget'] = allocated_budget
                cost_data['start_date'] = start_date
                cost_data.update(common_data)
                cost_data['index_id'] = f"""{cost_data['start_date']}-{cost_data['AccountId']}"""
                cost_data['timestamp'] = datetime.datetime.strptime(cost_data['start_date'], '%Y-%m-%d')
                cost_data['Month'] = datetime.datetime.strftime(cost_data['timestamp'], '%Y %b')
                cost_data['filter_date'] = f"{cost_data['start_date']}-{cost_data['Month'].split()[-1]}"
                cost_billing_data[index_id] = cost_data
            else:
                cost_billing_data[index_id].update(cost_data)
            if start_date:
                if 'Actual' not in cost_billing_data[index_id]:
                    cost_billing_data[index_id]['Actual'] = 0
                else:
                    if 'Forecast' not in cost_billing_data[index_id]:
                        cost_billing_data[index_id]['Forecast'] = 0
        return cost_billing_data

    def get_total_billing_accounts(self):
        """
        This method returns the data of all billing accounts
        """
        cost_billing_data = {}
        for profile in self.azure_operations.get_billing_profiles():
            profile = profile.as_dict()
            scope = profile.get('id')
            self.__common_data['AccountProfile'] = profile.get('display_name')
            cost_center = int(profile.get('display_name').split('(CC ')[-1].split(')')[0])
            self.__common_data['CostCenter'] = cost_center
            filters = {
                'grouping': [{'type':'Dimension', 'name': 'SubscriptionName'},
                             {'type': 'Dimension', 'name': 'SubscriptionId'}]
            }
            usage_data = self.cost_mgmt_operations.get_usage(scope=scope, **filters)
            subscription_ids = []
            if usage_data.get('rows'):
                subscription_ids = [[id[3], id[2]] for id in usage_data.get('rows')]
                self.get_data_from_costs(cost_data_rows=usage_data.get('rows'), cost_data_columns=usage_data.get('columns'), cost_type='Actual',
                                         cost_billing_data=cost_billing_data)
            for subscription in subscription_ids:
                query_filter = {
                    'filter': {'name': 'SubscriptionId', 'values': [subscription[0]]}}
                forecast_data = self.cost_mgmt_operations.get_forecast(scope=scope, **query_filter)
                if forecast_data and forecast_data.get('rows'):
                    self.get_data_from_costs(cost_data_rows=forecast_data.get('rows'),
                                             cost_data_columns=forecast_data.get('columns'), cost_type='Forecast',
                                             cost_billing_data=cost_billing_data, subscription_id=subscription[0],
                                             account_name=subscription[1], cost_center=cost_center)
            self.__common_data = self.get_common_data()
        return cost_billing_data

    @logger_time_stamp
    def collect_and_upload_cost_data(self):
        """
        This method collect and upload the data
        @return:
        """
        cost_billing_data = {}
        if not self.__total_account:
            scope = self.azure_operations.get_billing_profiles_list(subscription_id=self.azure_operations.subscription_id)
            if scope:
                usage_data = self.cost_mgmt_operations.get_usage(scope=scope)
                forecast_data = self.cost_mgmt_operations.get_forecast(scope=self.azure_operations.scope)
                if usage_data:
                    self.get_data_from_costs(cost_data_rows=usage_data.get('rows'), cost_data_columns=usage_data.get('columns'), cost_type='Actual', cost_billing_data=cost_billing_data)
                if forecast_data:
                    self.get_data_from_costs(cost_data_rows=forecast_data.get('rows'), cost_data_columns=forecast_data.get('columns'), cost_type='Forecast', cost_billing_data=cost_billing_data, subscription_id=self.azure_operations.subscription_id)
            else:
                logger.warning(f"No billing scopes found in the subscription: {self.azure_operations.subscription_id}")
        else:
            cost_billing_data = self.get_total_billing_accounts()
        if cost_billing_data:
            cost_billing_data = dict(sorted(cost_billing_data.items()))
            uploaded_cost_data = list(cost_billing_data.values())
            self.elastic_upload.es_upload_data(items=uploaded_cost_data, set_index='index_id')
            if not self.__total_account:
                upload_data = {
                    'cloud_alias_name': f'{self.azure_operations.account_name}-azure',
                    'cloud_name': 'AZURE',
                    'cloud_data': uploaded_cost_data
                }
                self.update_to_gsheet.update_data(cloud_data=upload_data)
        return list(cost_billing_data.values())

    def run(self):
        """
        This method runs the azure cost billing reports
        @return:
        """
        self.collect_and_upload_cost_data()



import boto3

from cloud_governance.cloud_resource_orchestration.clouds.aws.ec2.aws_monitor_tickets import AWSMonitorTickets
from cloud_governance.cloud_resource_orchestration.utils.common_operations import string_equal_ignore_case
from cloud_governance.main.environment_variables import environment_variables


class CroObject:
    """
    This class implements the CRO activities
    """

    def __init__(self, public_cloud_name: str):
        self.__public_cloud_name = public_cloud_name
        self.__environment_variables_dict = environment_variables.environment_variables_dict
        self.__run_active_regions = self.__environment_variables_dict.get('RUN_ACTIVE_REGIONS')
        self.__region = self.__environment_variables_dict.get('AWS_DEFAULT_REGION', '')

    def cost_over_usage(self):
        """
        This method returns the cost ove rusage object
        :return:
        :rtype:
        """
        if string_equal_ignore_case(self.__public_cloud_name, 'aws'):
            from cloud_governance.cloud_resource_orchestration.clouds.aws.ec2.cost_over_usage import CostOverUsage
            return CostOverUsage()
        elif string_equal_ignore_case(self.__public_cloud_name, 'azure'):
            from cloud_governance.cloud_resource_orchestration.clouds.azure.resource_groups.cost_over_usage import \
                CostOverUsage
            return CostOverUsage()

    def collect_cro_reports(self):
        """
        This method returns the cro reports collection object
        :return:
        :rtype:
        """
        if string_equal_ignore_case(self.__public_cloud_name, 'aws'):
            from cloud_governance.cloud_resource_orchestration.clouds.aws.ec2.collect_cro_reports import \
                CollectCROReports
            return CollectCROReports()
        elif string_equal_ignore_case(self.__public_cloud_name, 'azure'):
            from cloud_governance.cloud_resource_orchestration.clouds.azure.resource_groups.collect_cro_reports import \
                CollectCROReports
            return CollectCROReports()

    def monitor_tickets(self):
        """
        This method returns the cro monitor tickets object
        :return:
        :rtype:
        """
        if string_equal_ignore_case(self.__public_cloud_name, 'aws'):
            return AWSMonitorTickets()
        elif string_equal_ignore_case(self.__public_cloud_name, 'azure'):
            from cloud_governance.cloud_resource_orchestration.clouds.azure.resource_groups.azure_monitor_tickets\
                import AzureMonitorTickets
            return AzureMonitorTickets()

    def get_tag_cro_resources_object(self, region_name: str):
        """
        This method returns the tag cro resources object
        :param region_name:
        :type region_name:
        :return:
        :rtype:
        """
        if string_equal_ignore_case(self.__public_cloud_name, 'aws'):
            from cloud_governance.cloud_resource_orchestration.clouds.aws.ec2.tag_cro_instances import TagCROInstances
            return TagCROInstances(region_name=region_name)
        elif string_equal_ignore_case(self.__public_cloud_name, 'azure'):
            from cloud_governance.cloud_resource_orchestration.clouds.azure.resource_groups.tag_cro_resources import \
                TagCROResources
            return TagCROResources()

    def get_monitor_cro_resources_object(self, region_name: str):
        """
        This method returns the monitor cro resources object
        :param region_name:
        :type region_name:
        :return:
        :rtype:
        """
        if string_equal_ignore_case(self.__public_cloud_name, 'aws'):
            from cloud_governance.cloud_resource_orchestration.clouds.aws.ec2.monitor_cro_instances import MonitorCROInstances
            return MonitorCROInstances(region_name=region_name)
        elif string_equal_ignore_case(self.__public_cloud_name, 'azure'):
            from cloud_governance.cloud_resource_orchestration.clouds.azure.resource_groups.monitor_cro_resources import \
                MonitorCROResources
            return MonitorCROResources()

    def get_active_regions(self):
        """
        This method returns the regions to run cro
        :return:
        :rtype:
        """
        active_regions = []
        if string_equal_ignore_case(self.__public_cloud_name, 'aws'):
            if self.__run_active_regions:
                active_regions = [region.get('RegionName') for region in
                                  boto3.client('ec2').describe_regions()['Regions']]
            else:
                active_regions = [self.__region]
        elif string_equal_ignore_case(self.__public_cloud_name, 'azure'):
            active_regions = ['all']
        return active_regions



from cloud_governance.policy.helpers.azure.azure_policy_operations import AzurePolicyOperations


class UnUsedNatGateway(AzurePolicyOperations):
    """
    This class performs the azure unused nat gateway operations
    """

    RESOURCE_ACTION = "Delete"

    def __init__(self):
        super().__init__()
        self.__active_cluster_ids = self._get_active_cluster_ids()

    def __check_nat_gateway_metrics(self, resource_id: str):
        """
        This method returns bool by verifying nat metrics
        :param resource_id:
        :type resource_id:
        :return:
        :rtype:
        """
        metrics_data = self.monitor_operations.get_resource_metrics(resource_id=resource_id,
                                                                    metricnames='SNATConnectionCount',
                                                                    aggregation='Average')
        if metrics_data.get('value'):
            metrics_time_series_data = metrics_data.get('value', [])[0].get('timeseries', [])
            if metrics_time_series_data:
                for metric_time_frame in metrics_time_series_data:
                    for data in metric_time_frame.get('data'):
                        if data.get('average', 0) > 0:
                            return False
        return True

    def run_policy_operations(self):
        """
        This method returns the list of unused nat gateways
        :return:
        :rtype:
        """
        unused_nat_gateways = []
        nat_gateways = self.network_operations.describe_nat_gateways()
        for nat_gateway in nat_gateways:
            tags = nat_gateway.get('tags')
            cluster_tag = self._get_cluster_tag(tags=tags)
            cleanup_result = False
            if cluster_tag not in self.__active_cluster_ids:
                if self.__check_nat_gateway_metrics(resource_id=nat_gateway.get('id')):
                    cleanup_days = self.get_clean_up_days_count(tags=tags)
                    cleanup_result = self.verify_and_delete_resource(resource_id=nat_gateway.get('id'), tags=tags,
                                                                     clean_up_days=cleanup_days)
                    resource_data = self._get_es_schema(resource_id=nat_gateway.get('name'),
                                                        user=self.get_tag_name_from_tags(tags=tags, tag_name='User'),
                                                        skip_policy=self.get_skip_policy_value(tags=tags),
                                                        cleanup_days=cleanup_days, dry_run=self._dry_run,
                                                        name=nat_gateway.get('name'),
                                                        region=nat_gateway.get('location'),
                                                        cleanup_result=str(cleanup_result),
                                                        resource_action=self.RESOURCE_ACTION,
                                                        cloud_name=self._cloud_name,
                                                        resource_type=f"NatGateway: "
                                                                      f"{nat_gateway.get('sku', {}).get('name')}",
                                                        resource_state=nat_gateway.get('provisioning_state')
                                                        if not cleanup_result else "Deleted")
                    unused_nat_gateways.append(resource_data)
                else:
                    cleanup_days = 0
            else:
                cleanup_days = 0
            if not cleanup_result:
                self.update_resource_day_count_tag(resource_id=nat_gateway.get("id"), cleanup_days=cleanup_days,
                                                   tags=tags)

        return unused_nat_gateways



import os
import typeguard
from ast import literal_eval  # str to dict
import boto3  # regions

from cloud_governance.main.main_common_operations import run_common_policies
from cloud_governance.main.main_oerations.main_operations import MainOperations
from cloud_governance.main.run_cloud_resource_orchestration import run_cloud_resource_orchestration
from cloud_governance.policy.policy_operations.aws.cost_expenditure.cost_report_policies import CostReportPolicies
from cloud_governance.policy.policy_operations.azure.azure_policy_runner import AzurePolicyRunner
from cloud_governance.common.logger.logger_time_stamp import logger_time_stamp, logger
from cloud_governance.policy.policy_operations.aws.tag_cluster.run_tag_cluster_resouces import tag_cluster_resource, \
    remove_cluster_resources_tags
from cloud_governance.policy.policy_operations.aws.tag_non_cluster.run_tag_non_cluster_resources import tag_non_cluster_resource, \
    remove_tag_non_cluster_resource, tag_na_resources
from cloud_governance.policy.policy_operations.aws.tag_user.run_tag_iam_user import tag_iam_user, run_validate_iam_user_tags
from cloud_governance.policy.policy_operations.aws.zombie_cluster.run_zombie_cluster_resources import zombie_cluster_resource
from cloud_governance.policy.policy_operations.gcp.gcp_policy_runner import GcpPolicyRunner
from cloud_governance.policy.policy_operations.gitleaks.gitleaks import GitLeaks
from cloud_governance.policy.policy_operations.ibm.ibm_operations.ibm_policy_runner import IBMPolicyRunner
from cloud_governance.main.environment_variables import environment_variables
from cloud_governance.main.es_uploader import ESUploader
from cloud_governance.common.clouds.aws.s3.s3_operations import S3Operations
from cloud_governance.policy.policy_operations.aws.zombie_cluster.validate_zombies import ValidateZombies
from cloud_governance.policy.policy_operations.aws.zombie_non_cluster.zombie_non_cluster_polices import ZombieNonClusterPolicies


environment_variables_dict = environment_variables.environment_variables_dict
log_level = environment_variables_dict.get('log_level', 'INFO').upper()
logger.setLevel(level=log_level)


def get_custodian_policies(type: str = None):
    """
    This method return a list of policies name without extension, that can filter by type
    @return: list of custodian policies name
    """
    custodian_policies = []
    # path for debug only
    # policies_path = os.path.join(os.path.dirname(os.path.dirname(__file__)), 'policy')
    # policies_path working only inside the docker
    policies_path = os.path.join(os.path.dirname(__file__), 'policy', 'aws')
    for (dirpath, dirnames, filenames) in os.walk(policies_path):
        for filename in filenames:
            if not filename.startswith('__') and (filename.endswith('.yml') or filename.endswith('.py')):
                if not type:
                    custodian_policies.append(os.path.splitext(filename)[0])
                elif type and type in filename:
                    custodian_policies.append(os.path.splitext(filename)[0])
    return custodian_policies


@logger_time_stamp
@typeguard.typechecked
def run_policy(account: str, policy: str, region: str, dry_run: str):
    """
    This method run policy per region, first the custom policy and after custodian policy
    :return:
    """
    # policy Tag Cluster
    if policy == 'tag_resources':
        cluster_name = environment_variables_dict.get('resource_name',
                                                      '')  # environment_variables_dict.get('resource_name', '')
        mandatory_tags = environment_variables_dict.get('mandatory_tags', {})
        tag_operation = environment_variables_dict.get('tag_operation', '')
        if mandatory_tags:
            mandatory_tags = literal_eval(mandatory_tags)  # str to dict
        if tag_operation == 'delete':
            remove_cluster_resources_tags(region=region, cluster_name=cluster_name, input_tags=mandatory_tags)
        else:
            tag_cluster_resource(cluster_name=cluster_name, mandatory_tags=mandatory_tags, region=region,
                                 tag_operation=tag_operation)
    elif policy == 'validate_iam_user_tags':
        es_host = environment_variables_dict.get('es_host', '')
        es_port = environment_variables_dict.get('es_port', '')
        es_index = environment_variables_dict.get('es_index', '')
        validate_type = environment_variables_dict.get('validate_type', '')
        user_tags = environment_variables_dict.get('user_tags', {})
        if user_tags:
            user_tags = literal_eval(user_tags)
        run_validate_iam_user_tags(es_host=es_host, es_port=es_port, es_index=es_index, validate_type=validate_type,
                                   user_tags=user_tags)
    elif policy == 'validate_cluster':
        file_path = environment_variables_dict.get('file_path', '')
        file_name = environment_variables_dict.get('file_name', '')
        file_path = file_path + file_name
        validate_zombies = ValidateZombies(file_path=file_path, region=region)
        validate_zombies.read_csv()
    elif policy == 'tag_cluster':
        cluster_name = environment_variables_dict.get('resource_name', '')
        mandatory_tags = environment_variables_dict.get('mandatory_tags', {})
        tag_operation = environment_variables_dict.get('tag_operation', '')
        if mandatory_tags:
            mandatory_tags = literal_eval(mandatory_tags)  # str to dict
        if tag_operation == 'delete':
            remove_cluster_resources_tags(region=region, cluster_name=cluster_name, input_tags=mandatory_tags,
                                          cluster_only=True)
        else:
            tag_cluster_resource(cluster_name=cluster_name, mandatory_tags=mandatory_tags, region=region,
                                 tag_operation=tag_operation, cluster_only=True)
    elif policy == 'tag_iam_user':
        user_tag_operation = environment_variables_dict.get('user_tag_operation', '')
        file_name = environment_variables_dict.get('file_name', '')
        username = environment_variables_dict.get('username', '')
        remove_keys = environment_variables_dict.get('remove_tags', '')
        if remove_keys:
            remove_keys = literal_eval(remove_keys)
        tag_iam_user(user_tag_operation=user_tag_operation, file_name=file_name, remove_keys=remove_keys,
                     username=username)
    elif policy == 'zombie_cluster_resource':
        policy_output = environment_variables_dict.get('policy_output', '')
        resource = environment_variables_dict.get('resource', '')
        resource_name = environment_variables_dict.get('resource_name', '')
        cluster_tag = environment_variables_dict.get('cluster_tag', '')
        service_type = environment_variables_dict.get('service_type', '')
        if dry_run == 'no':  # delete
            zombie_result = zombie_cluster_resource(delete=True, region=region, resource=resource,
                                                    cluster_tag=cluster_tag, resource_name=resource_name,
                                                    service_type=service_type)
        else:  # default: yes or other
            zombie_result = zombie_cluster_resource(region=region, resource=resource, cluster_tag=cluster_tag,
                                                    resource_name=resource_name, service_type=service_type)
        if policy_output:
            s3operations = S3Operations(region_name=region)
            logger.info(s3operations.save_results_to_s3(policy=policy.replace('_', '-'), policy_output=policy_output,
                                                        policy_result=zombie_result))
    elif policy == 'tag_non_cluster':
        # instance_name = environment_variables_dict['resource_name']
        mandatory_tags = environment_variables_dict.get('mandatory_tags', {})
        tag_operation = environment_variables_dict.get('tag_operation', '')
        file_name = environment_variables_dict.get('file_name', '')
        if file_name:
            file_path = environment_variables_dict.get('file_path', '')
            if file_path:
                tag_na_resources(file_name=file_name, region=region, tag_operation=tag_operation, file_path=file_path)
            else:
                tag_na_resources(file_name=file_name, region=region, tag_operation=tag_operation)
        else:
            if mandatory_tags:
                mandatory_tags = literal_eval(mandatory_tags)  # str to dict
            # mandatory_tags['Name'] = instance_name
            if tag_operation == 'delete':
                remove_tag_non_cluster_resource(mandatory_tags=mandatory_tags, region=region, dry_run='no')
            else:
                tag_non_cluster_resource(mandatory_tags=mandatory_tags, region=region, tag_operation=tag_operation)
    elif policy == 'gitleaks':
        git_access_token = environment_variables_dict.get('git_access_token')
        git_repo = environment_variables_dict.get('git_repo')
        several_repos = environment_variables_dict.get('several_repos', '')
        policy_output = environment_variables_dict.get('policy_output', '')
        try:
            if several_repos == 'yes':
                git_leaks = GitLeaks(git_access_token=git_access_token,
                                     git_repo=git_repo,
                                     several_repos=True)
            else:
                git_leaks = GitLeaks(git_access_token=git_access_token,
                                     git_repo=git_repo)
            policy_result = git_leaks.scan_repo()
            logger.info(policy_result)
            if policy_output:
                s3operations = S3Operations(region_name=region)
                logger.info(s3operations.save_results_to_s3(policy=policy, policy_output=policy_output,
                                                            policy_result=policy_result))

        except Exception as err:
            logger.exception(f'BadCredentialsException : {err}')
    else:
        logger.exception(f'Missing Policy name: {policy}')
        raise Exception(f'Missing Policy name: {policy}')


def is_policy_aws():
    """
    This method true if the env contain AWS variables like , (AWS_ACCESS_KEY_ID and AWS_SECRET_ACCESS_KEY)
    or PUBLIC_CLOUD_NAME=AWS
    :return:
    :rtype:
    """
    aws_access_key = environment_variables_dict.get('AWS_ACCESS_KEY_ID', '')
    aws_secret_key = environment_variables_dict.get('AWS_SECRET_ACCESS_KEY', '')
    public_cloud_name = environment_variables_dict.get('PUBLIC_CLOUD_NAME', '')
    if aws_secret_key and aws_access_key:
        return True
    return public_cloud_name.lower() == 'aws'


@logger_time_stamp
def main():
    """
    This main run 2 processes:
    1. ES uploader
    2. Run policy
    :return: the action output
    """
    # environment variables - get while running the docker
    region_env = environment_variables_dict.get('AWS_DEFAULT_REGION', 'us-east-2')
    dry_run = environment_variables_dict.get('dry_run', 'yes')

    account = environment_variables_dict.get('account', '')
    policy = environment_variables_dict.get('policy', '')
    upload_data_es = environment_variables_dict.get('upload_data_es', '')
    es_host = environment_variables_dict.get('es_host', '')
    es_port = environment_variables_dict.get('es_port', '')
    es_index = environment_variables_dict.get('es_index', '')
    es_doc_type = environment_variables_dict.get('es_doc_type', '')
    bucket = environment_variables_dict.get('bucket', '')
    main_operations = MainOperations()
    response = main_operations.run()
    if not response:
        if environment_variables_dict.get('COMMON_POLICIES'):
            run_common_policies()
        elif environment_variables_dict.get('CLOUD_RESOURCE_ORCHESTRATION'):
            run_cloud_resource_orchestration()
        else:
            non_cluster_polices_runner = None
            is_non_cluster_polices_runner = policy in environment_variables_dict.get('aws_non_cluster_policies')
            if is_non_cluster_polices_runner:
                non_cluster_polices_runner = ZombieNonClusterPolicies()

            ibm_classic_infrastructure_policy_runner = None
            is_tag_ibm_classic_infrastructure_runner = policy in environment_variables_dict.get('ibm_policies')
            if not is_tag_ibm_classic_infrastructure_runner:
                if environment_variables_dict.get('PUBLIC_CLOUD_NAME') and environment_variables_dict.get('PUBLIC_CLOUD_NAME').upper() == 'IBM':
                    is_tag_ibm_classic_infrastructure_runner = policy in environment_variables_dict.get('cost_policies')
            if is_tag_ibm_classic_infrastructure_runner:
                ibm_classic_infrastructure_policy_runner = IBMPolicyRunner()

            is_cost_explorer_policies_runner = ''
            if environment_variables_dict.get('PUBLIC_CLOUD_NAME').upper() == 'AWS':
                cost_explorer_policies_runner = None
                is_cost_explorer_policies_runner = policy in environment_variables_dict.get('cost_policies')
                if is_cost_explorer_policies_runner:
                    cost_explorer_policies_runner = CostReportPolicies()

            is_azure_policy_runner = ''
            if environment_variables_dict.get('PUBLIC_CLOUD_NAME') and environment_variables_dict.get('PUBLIC_CLOUD_NAME').upper() == 'AZURE':
                azure_cost_policy_runner = None
                is_azure_policy_runner = policy in environment_variables_dict.get('cost_policies')
                if is_azure_policy_runner:
                    azure_cost_policy_runner = AzurePolicyRunner()

            is_gcp_policy_runner = ''
            if environment_variables_dict.get('PUBLIC_CLOUD_NAME') and environment_variables_dict.get('PUBLIC_CLOUD_NAME').upper() == 'GCP':
                gcp_cost_policy_runner = None
                is_gcp_policy_runner = policy in environment_variables_dict.get('cost_policies')
                if is_gcp_policy_runner:
                    gcp_cost_policy_runner = GcpPolicyRunner()

            @logger_time_stamp
            def run_non_cluster_polices_runner():
                """
                This method run the aws non-cluster policies
                @return:
                """
                non_cluster_polices_runner.run()

            def run_tag_ibm_classic_infrastructure_runner():
                """
                This method run the IBM policies
                @return:
                """
                ibm_classic_infrastructure_policy_runner.run()

            @logger_time_stamp
            def run_cost_explorer_policies_runner():
                """
                This method run the aws cost_explorer policies
                @return:
                """
                cost_explorer_policies_runner.run()

            @logger_time_stamp
            def run_azure_policy_runner():
                """
                This method run the azure policies
                @return:
                """
                azure_cost_policy_runner.run()

            @logger_time_stamp
            def run_gcp_policy_runner():
                """
                This method run the gcp policies
                """
                gcp_cost_policy_runner.run()

            # 1. ELK Uploader
            if upload_data_es:
                input_data = {'es_host': es_host,
                              'es_port': int(es_port),
                              'es_index': es_index,
                              'es_doc_type': es_doc_type,
                              'es_add_items': {'account': account},
                              'bucket': bucket,
                              'logs_bucket_key': 'logs',
                              's3_file_name': 'resources.json',
                              'region': region_env,
                              'policy': policy,
                              }
                elk_uploader = ESUploader(**input_data)
                elk_uploader.upload_to_es(account=account)
            # 2. POLICY
            elif is_non_cluster_polices_runner:
                run_non_cluster_polices_runner()
            elif is_tag_ibm_classic_infrastructure_runner:
                run_tag_ibm_classic_infrastructure_runner()
            elif is_cost_explorer_policies_runner:
                run_cost_explorer_policies_runner()
            elif is_azure_policy_runner:
                run_azure_policy_runner()
            elif is_gcp_policy_runner:
                run_gcp_policy_runner()
            else:
                if not policy:
                    logger.exception(f'Missing Policy name: "{policy}"')
                    raise Exception(f'Missing Policy name: "{policy}"')
                if region_env == 'all':
                    # must be set for boto3 client default region
                    # environment_variables_dict['AWS_DEFAULT_REGION'] = 'us-east-2'
                    ec2 = boto3.client('ec2')
                    regions_data = ec2.describe_regions()
                    for region in regions_data['Regions']:
                        # logger.info(f"region: {region['RegionName']}")
                        environment_variables_dict['AWS_DEFAULT_REGION'] = region['RegionName']
                        run_policy(account=account, policy=policy, region=region['RegionName'], dry_run=dry_run)
                else:
                    run_policy(account=account, policy=policy, region=region_env, dry_run=dry_run)


main()



from cloud_governance.common.utils.configs import CLOUDWATCH_METRICS_AVAILABLE_DAYS
from cloud_governance.common.utils.utils import Utils
from cloud_governance.policy.helpers.aws.aws_policy_operations import AWSPolicyOperations


class DatabaseIdle(AWSPolicyOperations):
    """
    This class performs the idle database operations
    """

    RESOURCE_ACTION = 'Delete'

    def __init__(self):
        super().__init__()

    def run_policy_operations(self):
        """
        This method returns the idle databases
        :return:
        :rtype:
        """
        idle_dbs = []
        dbs = self._rds_operations.describe_db_instances()
        for db in dbs:
            resource_id = db.get('DBInstanceIdentifier')
            create_date = db.get('InstanceCreateTime')
            tags = db.get('TagList', [])
            cluster_tag = self._get_cluster_tag(tags=tags)
            cleanup_result = False
            running_days = self.calculate_days(create_date=create_date)
            cleanup_days = 0
            resource_arn = db.get('DBInstanceArn', '')
            if Utils.greater_than(val1=running_days, val2=CLOUDWATCH_METRICS_AVAILABLE_DAYS) \
                    and not cluster_tag \
                    and self.is_database_idle(resource_id):
                cleanup_days = self.get_clean_up_days_count(tags=tags)
                cleanup_result = self.verify_and_delete_resource(resource_id=resource_id, tags=tags,
                                                                 clean_up_days=cleanup_days)
                unit_price = self._resource_pricing.get_rds_price(region_name=self._region,
                                                                  instance_type=db.get('DBInstanceClass'))
                resource_data = self._get_es_schema(resource_id=resource_id,
                                                    user=self.get_tag_name_from_tags(tags=tags, tag_name='User'),
                                                    skip_policy=self.get_skip_policy_value(tags=tags),
                                                    cleanup_days=cleanup_days, dry_run=self._dry_run,
                                                    name=db.get('DBName'),
                                                    region=self._region,
                                                    cleanup_result=str(cleanup_result),
                                                    resource_action=self.RESOURCE_ACTION,
                                                    cloud_name=self._cloud_name,
                                                    create_date=str(create_date),
                                                    resource_type=db.get('DBInstanceClass'),
                                                    unit_price=unit_price,
                                                    resource_state=db.get('DBInstanceStatus')
                                                    if not cleanup_result else "Deleted"
                                                    )
                idle_dbs.append(resource_data)
            if not cleanup_result:
                self.update_resource_day_count_tag(resource_id=resource_arn, cleanup_days=cleanup_days, tags=tags)

        return idle_dbs
